import { IIterator, IIterable } from '@lumino/algorithm';
import { Datastore, IServerAdapter, Schema, Table, Record } from '@lumino/datastore';
import { IDisposable } from '@lumino/disposable';
import { ISignal } from '@lumino/signaling';
import { Message } from '@lumino/messaging';
export declare type DatastoreFn = ((transaction: Datastore.Transaction) => void) | null;
/**
 * A patch store.
 */
export declare class TransactionStore {
    /**
     * Construct a new patch store.
     */
    constructor();
    /**
     * Returns the current undo stack
     */
    get undoStack(): string[];
    /**
     * Returns the current redo stack
     */
    get redoStack(): string[];
    /**
     * Add a transaction to the patch store.
     *
     * @param - the transaction to add to the store.
     *
     * @returns - whether it was successfully added.
     */
    add(transaction: Datastore.Transaction): boolean;
    /**
     * Get a transaction by id.
     *
     * @param transactionId - the id of the transaction.
     *
     * @returns - the transaction, or undefined if it can't be found.
     */
    get(transactionId: string): Datastore.Transaction | undefined;
    /**
     * Handle a transaction undo.
     *
     * @param transactionId - the ID of the transaction to undo.
     *
     * #### Notes
     * This has no effect on the content of the transaction. Instead, it
     * updates its undo count in the internal cemetery, determining whether
     * the transaction should be applied at any given time.
     */
    undo(transactionId: string): void;
    /**
     * Handle a transaction redo.
     *
     * @param transactionId - the ID of the transaction to redo.
     *
     * #### Notes
     * This has no effect on the content of the transaction. Instead, it
     * updates its undo count in the internal cemetery, determining whether
     * the transaction should be applied at any given time.
     */
    redo(transactionId: string): void;
    /**
     * Get the entire history for the transaction store.
     *
     * @returns - an array of transactions representing the whole history.
     */
    getHistory(): Datastore.Transaction[];
    get cemetery(): {
        [id: string]: number;
    };
    /**
     * Get the most recent transaction (newly added or redone).
     *
     * @returns - the transaction, or undefined if there aren't any.
     */
    getLastTransaction(): Datastore.Transaction | undefined;
    /**
     * Get the most recent undone transaction.
     *
     * @returns - the transaction, or undefined if there aren't any.
     */
    getLastUndo(): Datastore.Transaction | undefined;
    private _order;
    private _transactions;
    private _cemetery;
    private _undoStack;
    private _redoStack;
}
/**
 * A class providing minimal adapter functionality for undoing and redoing.
 *
 * ### Notes
 * Does not support broadcasting or multiple users.
 */
export declare class DummyAdapter implements IServerAdapter {
    /**
     * Construct a new adapter.
     *
     * @param transactionStore - the TransactionStore to use, or undefined.
     */
    constructor(transactionStore?: TransactionStore);
    /**
     * Adds each transaction to the TransactionStore.
     *
     * ### Notes
     * This method is supposed to broadcast each transaction to collaborators in
     * a "real" adapter. Here, it just acts as a convenient way to run some code
     * every time a new transaction occurs.
     */
    broadcast(transaction: Datastore.Transaction): void;
    /**
     * Whether the adapter has been disposed.
     */
    get isDisposed(): boolean;
    /**
     * Disposes of the resources held by the adapter.
     */
    dispose(): void;
    /**
     * A callback for when a remote transaction is recieved by the adapter (no op).
     */
    get onRemoteTransaction(): DatastoreFn;
    set onRemoteTransaction(fn: DatastoreFn);
    /**
     * A callback for when an undo request is recieved by the adapter.
     */
    get onUndo(): DatastoreFn;
    set onUndo(fn: DatastoreFn);
    /**
     * A callback for when a redo request is recieved by the adapter.
     */
    get onRedo(): DatastoreFn;
    set onRedo(fn: DatastoreFn);
    /**
     * Handles an undo message.
     *
     * @param id - the ID of the transaction to undo.
     *
     * ### Notes
     * This function should only be called by the Datastore.
     */
    undo(id: string): Promise<void>;
    /**
     * Handles a redo message.
     *
     * @param id
     *
     * ### Notes
     * This function should only be called by the Datastore.
     */
    redo(id: string): Promise<void>;
    private _isDisposed;
    private _transactionStore;
    private _onRemoteTransaction;
    private _onUndo;
    private _onRedo;
}
/**
 * A convenient Datastore wrapper for non-collaborative work where a local
 * database with undo, redo, or serialization functionality is needed.
 */
export declare class Litestore implements IDisposable, IIterable<Table<Schema>> {
    /**
     * Constructs a new Litestore.
     *
     * @param options - The options for creating the Litestore. The adapter will
     * be overwritten by a new DummyAdapter.
     */
    constructor(options: Datastore.IOptions);
    /**
     * The unique id of the store.
     */
    get id(): number;
    /**
     * A signal emitted when changes are made to the store.
     *
     * ### Notes
     * The payload represents the sert of local changes that were made
     * to bring the store to its current state.
     */
    get changed(): ISignal<Datastore, Datastore.IChangedArgs>;
    /**
     * Whether the store has been disposed.
     */
    get isDisposed(): boolean;
    /**
     * Whether a transaction is currently in progress.
     */
    get inTransaction(): boolean;
    /**
     * The current version of the store.
     *
     * ### Notes
     * This version is automatically increased for each transaction to
     * the store. However, it might not increase linearly
     * (i.e. it might make jumps).
     */
    get version(): number;
    /**
     * The dummy adapter for the store.
     */
    get adapter(): IServerAdapter;
    /**
     * The transaction cemetery used by the TransactionStore.
     */
    get cemetery(): {
        [id: string]: number;
    };
    get transactionStore(): TransactionStore;
    /**
     * Get a transaction by id.
     *
     * @param transactionId - the id of the transaction.
     *
     * @returns - the transaction, or undefined if it can't be found.
     */
    getTransaction(transactionId: string): Datastore.Transaction | undefined;
    /**
     * Get the entire history for the transaction store.
     *
     * @returns - an array of transactions representing the whole history.
     */
    getHistory(): Datastore.Transaction[];
    /**
     * Get the most recent transaction (newly added or redone).
     *
     * @returns - the transaction, or undefined if there aren't any.
     */
    getLastTransaction(): Datastore.Transaction | undefined;
    /**
     * Get the most recent undone transaction.
     *
     * @returns - the transaction, or undefined if there aren't any.
     */
    getLastUndo(): Datastore.Transaction | undefined;
    /**
     * Begin a new transaction in the store.
     *
     * @returns - the id of the new transaction.
     *
     *  @throws - an exception if a transaction is already in progress.
     *
     * #### Notes
     * This will allow the state of the store to be mutated
     * thorugh the `update` method on the individual tables.
     *
     * After the updates are completed, `endTransaction` should
     * be called.
     */
    beginTransaction(): string;
    /**
     * Completes a transaction.
     *
     * @returns - the ID of the last transaction, or undefined if it was empty.
     *
     * #### Notes
     * This completes a transaction previously started with
     * `beginTransaction`. If a change has occurred, the
     * `changed` signal will be emitted.
     */
    endTransaction(): string | undefined;
    /**
     * Get the table for a particular schema.
     *
     * @param schema - the schema of interest.
     *
     * @returns - the table for the specified schema.
     *
     * @throws - an exception if no table exists for the given schema.
     */
    get<S extends Schema>(schema: S): Table<S>;
    /**
     * Create an iterator over all the tables of the datastore.
     *
     * @returns - an iterator.
     */
    iter(): IIterator<Table<Schema>>;
    /**
     * Handle a message using the Datastore.
     */
    processMessage(msg: Message): void;
    /**
     * Disposes of the resources held by the store.
     */
    dispose(): void;
    /**
     * Undo a transaction that was previoulsy applied.
     *
     * @param transactionId - the ID of the transaction to undo, or undefined
     * to undo the last transaction.
     *
     * @returns - a promise which resolves when the action is complete.
     *
     * @throws - an exception if `undo` is called during a transaction.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    undo(transactionId?: string): Promise<void>;
    /**
     * Redo a transaction that was previously applied.
     *
     * @param transactionId - the ID of the transaction to redo, or undefined
     * to redo the last transaction.
     *
     * @returns - a promise which resolves when the action is complete.
     *
     * @throws - an exception if `undo` is called during a transaction.
     *
     * #### Notes
     * If changes are made, the `changed` signal will be emitted before
     * the promise resolves.
     */
    redo(transactionId?: string): Promise<void>;
    /**
     * Serialize the state of the store to a string.
     *
     * @returns - the serialized state.
     */
    toString(): string;
    /**
     * Get a given table by its location.
     *
     * @param loc - the table location ({schema}).
     *
     * @returns - the table.
     */
    getTable<S extends Schema>(loc: Datastore.TableLocation<S>): Table<S>;
    /**
     * Get a record by its location.
     *
     * @param loc - the record location ({schema, record}).
     *
     * @returns - the record.
     */
    getRecord<S extends Schema>(loc: Datastore.RecordLocation<S>): Record.Value<S> | undefined;
    /**
     * Get a field by its location.
     *
     * @param loc - the field location ({schema, record, field}).
     *
     * @returns - the field.
     */
    getField<S extends Schema, F extends keyof S['fields']>(loc: Datastore.FieldLocation<S, F>): S['fields'][F]['ValueType'];
    /**
     * Update a table.
     *
     * @param loc - the table location ({schema}).
     *
     * @param update - the update to the table.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    updateTable<S extends Schema>(loc: Datastore.TableLocation<S>, update: Table.Update<S>): void;
    /**
     * Update a record.
     *
     * @param loc - the record location ({schema, record}).
     *
     * @param update - the update to the record.
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    updateRecord<S extends Schema>(loc: Datastore.RecordLocation<S>, update: Record.Update<S>): void;
    /**
     * Update a field in a table.
     *
     * @param loc - the field location ({schema, record, field}).
     *
     * @param update - the update to the field (new field value).
     *
     * #### Notes
     * This does not begin a transaction, so usage of this function should be
     * combined with `beginTransaction`/`endTransaction`, or `withTransaction`.
     */
    updateField<S extends Schema, F extends keyof S['fields']>(loc: Datastore.FieldLocation<S, F>, update: S['fields'][F]['UpdateType']): void;
    /**
     * Listen to changes in a table. Changes to other tables are ignored.
     *
     * @param loc - the table location ({schema}).
     *
     * @param slot - a callback function to invoke when the table changes.
     *
     * @returns - an `IDisposable` that can be disposed to remove the listener.
     */
    listenTable<S extends Schema>(loc: Datastore.TableLocation<S>, slot: (source: Datastore, args: Table.Change<S>) => void, thisArg?: any): IDisposable;
    /**
     * Listen to changes in a record. Changes to other records are ignored.
     *
     * @param loc - the record location ({schema, record}).
     *
     * @param slot - a callback function to invoke when the record changes.
     *
     * @returns - `IDisposable` that can be disposed to remove the listener.
     */
    listenRecord<S extends Schema>(loc: Datastore.RecordLocation<S>, slot: (source: Datastore, args: Record.Change<S>) => void, thisArg?: any): IDisposable;
    /**
     * Listen to changes in a field. Changes to other fields are ignored.
     *
     * @param loc - the table location ({schema, record, field}).
     *
     * @param slot - a callback function to invoke when the field changes.
     *
     * @returns - an `IDisposable` that can be disposed to remove the listener.
     */
    listenField<S extends Schema, F extends keyof S['fields']>(loc: Datastore.FieldLocation<S, F>, slot: (source: Datastore, args: S['fields'][F]['ChangeType']) => void, thisArg?: any): IDisposable;
    /**
     * A helper function to wrap an update to the datastore in calls to
     * `beginTransaction` and `endTransaction`.
     *
     * @param update: - a function that performs the update on the datastore.
     *   The function is called with a transaction id string, in case the
     *   user wishes to store the transaction ID for later use.
     *
     * @returns - the transaction ID.
     *
     * #### Notes
     * If the datastore is already in a transaction, this does not attempt
     * to start a new one, and returns an empty string for the transaction
     * id. This allows for transactions to be composed a bit more easily.
     */
    withTransaction(update: (id: string) => void): string;
    private _transactionStore;
    private _adapter;
    private _dataStore;
    private _isDisposed;
}
