import { DSVModel } from '@jupyterlab/csvviewer';
import { MutableDataModel, DataModel, SelectionModel } from '@lumino/datagrid';
import { Litestore } from './litestore';
import { DSVEditor } from './widget';
import { Signal } from '@lumino/signaling';
import { ListField } from '@lumino/datastore';
export declare class EditorModel extends MutableDataModel {
    constructor(options: DSVModel.IOptions);
    /**
     * A boolean that determins whether the ghosts are visible to the grid.
     */
    get ghostsRevealed(): boolean;
    set ghostsRevealed(value: boolean);
    /**
     * The model which holds the string containing the contents of the file.
     */
    get model(): DSVModel;
    /**
     * A signal that emits when data is set to notify the Editor.
     */
    get onChangedSignal(): Signal<this, DSVEditor.ModelChangedArgs>;
    /**
     * The datastore used by the this class to read properties.
     */
    get litestore(): Litestore;
    /**
     * The setter used by parent classes to intialize the litestore.
     */
    set litestore(value: Litestore);
    /**
     * An array holding the data type for each column.
     * Current types are 'string' (generic), 'integer',
     * 'number', 'boolean' (see Private.boolyCheck),
     * and date.
     */
    get dataTypes(): Array<DataModel.Metadata>;
    set dataTypes(value: Array<DataModel.Metadata>);
    /**
     * The grid's current number of rows by region.
     * NOTE: we add one so that a ghost row appears.
     *
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * The grid's current number of columns by region.
     *
     * Note: the UI components use this method to get the column count
     * so it should reflect the grid's columns.
     * NOTE: we add 1 so that a ghost column appears.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * The grid's current number of rows. TOTAL, NOT BY REGION.
     */
    get totalRows(): number;
    /**
     * The grid's current number of columns. TOTAL, NOT BY REGION.
     *
     * Notes: This is equivalent to columnCount('body')
     */
    get totalColumns(): number;
    /**
     * A boolean value which should be set to true
     * when data formatting is toggled on.
     */
    get isDataFormatted(): boolean;
    set isDataFormatted(bool: boolean);
    get isDataFormattedChanged(): Signal<this, null>;
    /**
     * Computes the metadata at a location.
     */
    metadata(region: DataModel.CellRegion, row: number, column: number): DataModel.Metadata;
    /**
     * Computes the metadata of the entire data grid.
     */
    resetMetadata(): Array<DataModel.Metadata>;
    /**
     * This function is called by the datagrid to fill in values. It is called many times
     * and so should be efficient.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * The method for setting data in a single cell.
     */
    setData(region: DataModel.CellRegion, row: number, column: number, values: any, rowSpan?: number, columnSpan?: number, update?: DSVEditor.ModelChangedArgs | null): boolean;
    /**
     * The method for setting data in a range of cells.
     */
    bulkSetData(rowArray: Array<number>, columnArray: Array<number>, value: string): DSVEditor.ModelChangedArgs;
    /**
     * Add rows to the Editor.
     */
    addRows(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Add columns to the editor.
     */
    addColumns(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Add rows to the editor.
     */
    removeRows(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Remove rows from the editor.
     */
    removeColumns(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Move rows in the grid.
     */
    moveRows(region: DataModel.CellRegion, start: number, end: number, span: number): DSVEditor.ModelChangedArgs;
    /**
     * Handles the computations involved in moving rows in the grid.
     */
    rowMapSplice(rowMap: ListField.Value<number>, start: number, end: number, span: number): ListField.Splice<number>[];
    /**
     * Move columns in the grid.
     */
    moveColumns(region: DataModel.CellRegion, start: number, end: number, span: number): DSVEditor.ModelChangedArgs;
    /**
     * Handles computations associated with moving columns in the grid.
     */
    columnMapSplice(columnMap: ListField.Value<number>, start: number, end: number, span: number): ListField.Splice<number>[];
    /**
     * Clear a region of cells in the grid.
     */
    clearCells(region: DataModel.CellRegion, selection: SelectionModel.Selection): DSVEditor.ModelChangedArgs;
    /**
     * Clear rows in the grid.
     */
    clearRows(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Clear columns in the grid.
     */
    clearColumns(region: DataModel.CellRegion, start: number, span?: number): DSVEditor.ModelChangedArgs;
    /**
     * Cut a selection of cells.
     * NOTE: this method both copies the cells to the _clipboard property and clears them
     * from the region.
     */
    cut(region: DataModel.CellRegion, startRow: number, startColumn: number, endRow: number, endColumn: number): DSVEditor.ModelChangedArgs;
    /**
     * Copies a selection of data to the local propery _clipboard.
     * NOTE: this does not copy to the system clipboard. For this use the DataGrid method
     * copyToClipboard
     */
    copy(region: DataModel.CellRegion, startRow: number, startColumn: number, endRow: number, endColumn: number): void;
    /**
     * Paste a selection of cells onto the grid.
     * NOTE: this method first checks if we have data stored in our local clipboard. It
     * only checks for the data parameter if the local clipboard is empty.
     */
    paste(region: DataModel.CellRegion, row: number, column: number, data?: string | null): DSVEditor.ModelChangedArgs;
    /**
     * Emits the change which undoes the change passed in.
     */
    emitOppositeChange(update: DSVEditor.GridState): void;
    /**
     * Emits the change which is passed in as an argument.
     */
    emitCurrentChange(change: DataModel.ChangedArgs): void;
    /**
     * Returns the serializes string with the changes added.
     */
    updateString(): string;
    /**
     * Produces two arrays inverseRowMap and inverseColumnMap
     * such that for each x in the rowMap, inverseRowMap[x] ===
     * rowMap.indexOf(x). The same relationship holds for
     * inverseColumnMap and columnMap.
     *
     * ## Notes
     * An analogy to consider is that the changes we have made to the row and column
     * map are like shuffling a deck of cards. Now, we are unsuffling
     * by performing the opposite shuffle starting with the last shuffle
     * that was made. The only difference to note is that we are performing
     * an *arg-sort*, so we actually begin this process with a sorted deck.
     *
     * ### Further notes:
     * Though the row map and column map may change size in the process of mutations,
     * there is a way to translate these mutations into permutations on a larger array
     * that preserves uniqueness. What we are undoing is these permutations, which achieves an
     * argsort.
     */
    private _invertMaps;
    /**
     * The data grid indexes the rows and columns on a per-region basis. The DSVModel
     * treats the column header as the 0th row and counts up from there. This method
     * translates one to the other.
     */
    private _absoluteIndex;
    /**
     * translate from our unique row ID to the Grid's standard
     */
    private _regionIndex;
    /**
     * Processes updates from the DSVModel.
     */
    private _receiveModelSignal;
    /**
     * Adds new rows coming in from the asynchronous parsing of string by the DSVModel.
     */
    private _assimilateNewRows;
    /**
     * The total rows currently stored in the DSVModel.
     */
    private _stringRows;
    /**
     * The total columns currently stored in the DSVModel.
     */
    private _stringColumns;
    /**
     * Computes the offset index at the end of a given row (note: row delimeter not included).
     * NOTE: For the purposes of this function we care about the number of rows in the raw string.
     */
    private _rowEnd;
    private _openSlice;
    /**
     * Builds the slicing pattern that needs to be applied to every row in the model
     * @param columnMap
     * @returns The SlicePattern containing a list of buffers (arrays of delimiters) and slices (indexes to slice the original string on)
     */
    private _columnSlicePattern;
    private _performMacroSlice;
    private _serializer;
    /**
     * Returns a blank row with the correct numbers of columns and correct delimiters
     * @param model The DSV model being used
     * @param row The index of the row being inserted (determines whether to add a row delimiter or not)
     */
    private _blankRow;
    private _peformMicroSlice;
    private _clipboard;
    private _litestore;
    private _model;
    private _rowsAdded;
    private _columnsAdded;
    private _onChangeSignal;
    private _rowsRemoved;
    private _columnsRemoved;
    private _saving;
    private _ghostsRevealed;
    private _isDataFormatted;
    private _isDataFormattedChanged;
    private _dataTypes;
}
export declare type MapUpdate = 'add' | 'remove' | 'move' | 'clear';
export declare type SlicePattern = {
    buffers: Array<string>;
    slices: Array<Array<number>>;
};
